#!/usr/bin/ruby -Ks
#
#  YKLook
#        Copyright (C) 2009-2010 YorozuKotowariYa Ltd.


if ARGV.size() != 1
	STDERR.puts("need a filename")
	exit
end


YK_CLASSES = [ 'RuntimeError',
				'YKLookError',
				'YKLookErrorMain',
				'YKLookErrorUser',
				'YKLookErrorSwing' ]


def mTrimBlank( prmSource )
	idx = 0
	while idx < prmSource.size()
		if prmSource[idx] != 0x20 and prmSource[idx] != 0x09 and
			prmSource[idx] != 0x0d and prmSource[idx] != 0x0a
			if prmSource[idx] != 0x81
				break
			end
			if (idx + 1) >= prmSource.size()
				STDERR.puts('ill-kanji')
				STDERR.puts('str=(' + prmSource + ')')
				raise
			end
			if prmSource[idx+1] != 0x40
				break
			end
			idx += 1
		end
		idx += 1
	end
	if idx >= prmSource.size()
		return ''
	end
	j = prmSource.size() - 1
	while j >= 0
		if prmSource[j] != 0x20 and prmSource[j] != 0x09 and
			prmSource[j] != 0x0d and prmSource[j] != 0x0a
			if j < 1
				break
			end
			if prmSource[j-1] != 0x81 or prmSource[j] != 0x40
				break
			end
			j -= 1
		end
		j -= 1
	end
	if j < 0
		STDERR.puts('error fatal')
		exit
	end
	return prmSource[idx..j]
end

def mScanLines( prmLines )
	hash = { }
	prmLines.each() do |str|
		if str.length() >= 1 and str[0,1] == '#'
			next
		end
		ary = mTrimBlank( str ).split(' ')
		if ary.size() > 0
			if ary.size() < 3
				STDERR.puts('ill-fmt too few words')
				exit
			end
			if not YK_CLASSES.include?( ary[0] )
				STDERR.puts('ill-fmt unknown classname ' + ary[0])
				exit
			end
			val = hash[ ary[0] ]
			if not val
				val = { }
				hash[ ary[0] ] = val
			end
			if val[ ary[1] ]
				STDERR.puts('ill-fmt dup msg')
				exit
			end
			val[ ary[1] ] = ary[2..-1].join(' ')
		end
	end
	return hash
end
def mScanToMsg( prmHash )
	hash = { }
	prmHash.each() do |cls,val|
		val.each() do |msg,str|
			wrk = hash[msg]
			if not wrk
				wrk = { }
				hash[msg] = wrk
			end
			if wrk[cls]
				STDERR.puts('ill-fmt dup cls')
				exit
			end
			wrk[cls] = str
		end
	end
	return hash
end
def mCheckMsg( prmHash )
	prmHash.each() do |msg,val|
		fst = true
		obj = nil
		val.values().each() do |str|
			if fst
				fst = false
				obj = str
			else
				if obj != str
					STDERR.puts('ill-fmt str')
					exit
				end
			end
		end
	end
end

def mPutHeader()
	print <<"EOF"
<html lang=ja>
<head>
<meta http-equiv="Content-Type" content="text/html; charset=shift_jis">
<title> YKLook  G[̉EfobO  G[bZ[W </title>
<style type="text/css">
<!--  
.yk-leftmargin  { margin-left: 30px }
  -->
</style>
</head>




<!--  
                    ێS҂ւ̓`

                    ̃t@C𒼐ڕҏW_I@ij
  -->





<body bgcolor="#ffffff">
<table width="100%" border=0 cellpadding=0 cellspacing=0>
  <tr>
    <td>
      <a href="../index.html"> YKLook </a> 
      <a href="index.html"> G[̉EfobO </a> 
      <a name="top"> G[bZ[W </a>
    </td>
    <td align=right>
      <a href="index.html" title="G[̉EfobO"> O </a>
      <!-- kanji sp --> @ <!-- kanji sp -->
      <a href="deb-logfile.html" title="Ot@C">  </a>
    </td>
  </tr>
</table>
EOF
end
def mPutLinkBar()
	print <<"EOF"
<table bgcolor="#ccffcc" width="100%" border=0 cellpadding=0 cellspacing=0>
  <tr nowrap>
    <td>
      <!-- kanji sp --> @ <!-- kanji sp -->
    </td>
    <td> &nbsp; </td>
  </tr>
</table>
EOF
end
def mPutFooter()
	print <<"EOF"
<table width="100%" border=0 cellpadding=0 cellspacing=0>
  <tr>
    <td>
      <a href="../index.html"> YKLook </a> 
      <a href="index.html"> G[̉EfobO </a> 
      <a href="#top" title="̃y[W̐擪"> G[bZ[W </a>
    </td>
    <td align=right>
      <a href="index.html" title="G[̉EfobO"> O </a>
      <!-- kanji sp --> @ <!-- kanji sp -->
      <a href="deb-logfile.html" title="Ot@C">  </a>
    </td>
  </tr>
</table>
<br>
<hr>
<center>
  <span title="iLj낸Ƃ">
    Copyright (c) 2003-2010 YorozuKotowariYa Ltd.
  </span>
</center>

<!-- 

Copyright (c) 2003-2010 YorozuKotowariYa Ltd.
iLj낸Ƃ

designed by Hatashima Koji
written by Hatashima Koji
݌vF_
F_

 -->

</body>
</html>
EOF
end
def mPutSubTitle( prmLinkLabel, prmDisplayLabel )
	print <<"EOF"
<table bgcolor="#dddddd" width="100%" cellpadding=4>
<tr><td id="#{prmLinkLabel}"> #{prmDisplayLabel} </td></tr>
</table>
EOF
end

def mPutFromClass( prmHash )
	YK_CLASSES.each() do |cls|
		mPutSubTitle( cls, ('<code>' + cls + '</code>') )
		puts('<br />')
		puts('<div class="yk-leftmargin">')
		puts('<table border=1 cellpadding=4 cellspacing=0>')
		puts('  <tr>')
		puts('    <th nowrap> bZ[W </th>')
		puts('    <th>  </th>')
		puts('  </tr>')
		val = prmHash[cls]
		val.keys().sort().each() do |msg|
			str = val[msg]
			puts('  <tr>')
			puts('    <td nowrap> <code> ' + msg + ' </code> </td>')
			puts('    <td> ' + str + ' </td>')
			puts('  </tr>')
		end
		puts('</table>')
		puts('</div>')
		puts('<br />')
		puts('<br />')
		puts('')
		puts('')
	end
end
def mPutFromMessage( prmHash )
	mPutSubTitle( 'msg-abc', 'bZ[Ŵ`ab' )
	puts('<br />')
	puts('<div class="yk-leftmargin">')
	puts('<table border=1 cellpadding=4 cellspacing=0>')
	puts('  <tr>')
	puts('    <th nowrap> bZ[W </th>')
	puts('    <th>  </th>')
	puts('  </tr>')
	prmHash.keys().sort().each() do |msg|
		val = prmHash[ msg ]
		str = val[ val.keys()[0] ]
		puts('  <tr>')
		puts('    <td nowrap> <code> ' + msg + ' </code> </td>')
		puts('    <td> ' + str + ' </td>')
		puts('  </tr>')
	end
	puts('</table>')
	puts('</div>')
	puts('<br />')
	puts('<br />')
	puts('')
	puts('')
end
def mPutSecondLink()
	print <<'EOF'
<br />
<h1> G[bZ[W </h1>
<br />
ł YKLook o͂G[bZ[WɂċLqĂ܂B<br />
<ul>
  <li> ONX
    <ul>
      <li> <a href="#RuntimeError"    > <code> RuntimeError     </code> </a>
      <li> <a href="#YKLookError"     > <code> YKLookError      </code> </a>
      <li> <a href="#YKLookErrorMain" > <code> YKLookErrorMain  </code> </a>
      <li> <a href="#YKLookErrorUser" > <code> YKLookErrorUser  </code> </a>
      <li> <a href="#YKLookErrorSwing"> <code> YKLookErrorSwing </code> </a>
    </ul>
  <li> <a href="#msg-abc"> bZ[Ŵ`ab </a>
</ul>
<br />
<br />


EOF
end
def mDumpHash( prmHash )
	prmHash.each() do |key,val|
		puts( 'key=(' + key + ')-------------------------------------------' )
		val.each() do |msg,str|
			puts( 'msg=(' + msg + ') str=(' + str + ')' )
		end
		puts( 'end=(' + key + ')-------------------------------------------' )
	end
end


lines = File.readlines( ARGV[0] )
hash_cls = mScanLines( lines )
hash_msg = mScanToMsg( hash_cls )
mCheckMsg( hash_msg )

mPutHeader()
mPutLinkBar()
mPutSecondLink()
mPutFromClass( hash_cls )
mPutFromMessage( hash_msg )
puts('<br /> <br />')
mPutLinkBar()
mPutFooter()


# mDumpHash( hash_msg )


#
# end
